/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Juha Ristolainen <juha.ristolainen@codemancers.fi>
 *              Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 *              Nick Leppänen Larsson <frals@frals.se>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import QtQuick.XmlListModel 2.0
import "../common/ClockUtils.js" as Utils
import "../common/Constants.js" as Constants
import "../common"

Page {
    id: clockPage;

    // Property to hold the formatted time string to show on the screen
    property string currentTimeFormatted
    property real currentTimeStamp
    property real diff: new Date().getTimezoneOffset()

    Component.onCompleted: {
        Utils.log("ClockPage loaded");
        updateTime();
    }

    function onTimerUpdate(now) {
        currentTimeFormatted = Qt.formatTime(new Date(currentTimeStamp + (diff * 60000)), "hh:mm")
        now.setMinutes(now.getMinutes() + diff)
        clockFace.timeChanged(now)
        if (now.getUTCSeconds() == 0) {
            updateTime();
        }
    }

    function updateTime() {
        var now = new Date();
        currentTimeStamp = now.getTime();
    }

    function getTimeDifference(data) {
        var worldTime = data.split(' ')[1].split(':'),
            hoursWT = parseInt(worldTime[0]),
            minutesWT = parseInt(worldTime[1]),
            now = new Date();
        return ((hoursWT - now.getHours()) * 60 + (minutesWT - now.getMinutes()));
    }

    WorldClockModel {
        id: worldModel;

        Component.onCompleted: {
            _loadDB();
            diff = worldModel.timeDiff;
            currentLocation.text = worldModel.city;
            currentTimeFormatted = Qt.formatTime(new Date(currentTimeStamp + (diff * 60000)), "hh:mm")
        }
    }

    AnimationContainer {
        id: clockAnimationContainer

        initYPos: units.gu(0);
        finalYPos: -listWorldClocks.height;

        anchors.fill: parent

        // Label to show the current time
        Rectangle {
            id: labelContainer;

            z: clockFace.z + 1;
            width: clockFace.innerDimension; height: width;
            anchors.centerIn: clockFace;
            color: "transparent"

            Label {
                id: currentTimeLabel
                objectName: "currentTimeLabel"

                anchors.centerIn: parent
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                fontSize: "x-large"
                color: Constants.brightWhite;

                text: currentTimeFormatted
            }
        }

        // Qml Element to draw the analogue clock face along with its hour, minute and second hands.
        AnalogClockFace {
            id: clockFace

            anchors { top: parent.top; topMargin: units.gu(17); horizontalCenter: parent.horizontalCenter }
            Component.onCompleted: easterEggCircle.reloadSunRiseModel()

            onClicked: {
                if (easterEggCircle.isReady == XmlListModel.Ready) {
                    clockFace.state == "" ? clockFace.state = "easteregg" : clockFace.state = "";
                } else {
                    Utils.log("Sunrise/Sunset times not yet loaded.")
                }
            }

            EasterEgg {
                id: easterEggCircle
                anchors.centerIn: parent
                width: clockFace.innerDimension; height: width;
                radius: width / 2;
            }

            states: [
                State {
                    name: "easteregg"
                    PropertyChanges { target: easterEggCircle; opacity: 1; }
                    PropertyChanges { target: currentTimeLabel; opacity: 0; }
                    // TODO: Do some caching of data here. Store data locally to reduce API calls.
                    PropertyChanges { target: easterEggCircle; sunriseLabel: easterEggCircle.getSunTime(easterEggCircle.model.get(0).sunrise); }
                    PropertyChanges { target: easterEggCircle; sunsetLabel: easterEggCircle.getSunTime(easterEggCircle.model.get(0).sunset); }
                },
                State {
                    name: ""
                    PropertyChanges { target: currentTimeLabel; opacity: 1; }
                    PropertyChanges { target: easterEggCircle; opacity: 0; }
                }
            ]
        }

        Column {
            id: savedWorldClock

            height: childrenRect.height;
            anchors { left:parent.left; right:parent.right; top: clockFace.bottom; topMargin: units.gu(12)}

            ListItem.ThinDivider {}

            ListItem.Header { text: i18n.tr("Current Location") }

            ListItem.SingleValue {
                id: currentLocation
                text: "UTC"
                value: Qt.formatTime(new Date(currentTimeStamp + (diff * 60000)), "hh:mm A");
            }

            ListItem.Header { text: i18n.tr("World") }

            ListView {
                id: listWorldClocks

                anchors { left: parent.left; right: parent.right }
                height: units.gu(30)
                model: worldModel
                currentIndex: -1

                delegate: ListItem.Standard {
                    text: cityName;

                    Label {
                        fontSize: "medium"
                        text: Qt.formatTime(new Date(currentTimeStamp + (rawOffSet * 60000)), "hh:mm A");
                        anchors { verticalCenter: parent.verticalCenter; right: parent.right; rightMargin: units.gu(2) }
                    }

                    selected: listWorldClocks.currentIndex == index
                    removable: true

                    backgroundIndicator: SwipeDelete {
                        id: swipeDelete
                        state: swipingState
                    }

                    // TODO: Implement different based actions based on the swipe direction.
                    onItemRemoved: {
                        worldModel.removePreset(index);
                    }

                    onClicked: {
                        currentLocation.text = cityName;
                        diff = rawOffSet;
                        worldModel.appendCurrentLocation(cityName, diff);
                    }
                }
            }
        }
    }

    WorldClock {
        id: worldClockList;

        anchors { left: parent.left; right: parent.right; top: parent.top; topMargin: units.gu(2) }

        // Xml model to retrieve timezone of searched city
        XmlListModel {
            id: cityDetailsModel;

            source: worldClockList.getCityTimezoneUrl(worldClockList.lat, worldClockList.lng)
            query: "/geonames/timezone"

            onStatusChanged: {
                if(status == XmlListModel.Ready && worldClockList.city != "null") {
                    worldModel.appendPreset(worldClockList.city, getTimeDifference(cityDetailsModel.get(0).time));
                }
            }

            XmlRole { name: "time"; query: "time/string()" }
        }

        onClicked: {
            lat = searchModel.get(index).lat;
            lng = searchModel.get(index).lng;
            city = searchModel.get(index).city;
            cityDetailsModel.reload()
            clockPage.state = "NORMAL"
            worldClockList.clearUserSearch();
        }
    }

    states: [
        State {
            name: "ADDCITY"
            PropertyChanges { target: worldClockList; visible: true }
            PropertyChanges { target: clockAnimationContainer; visible: false }
            PropertyChanges { target: addCity; visible: false }
        },

        State {
            name: "NORMAL"
            PropertyChanges { target: worldClockList; visible: false }
            PropertyChanges { target: clockAnimationContainer; visible: true }
            PropertyChanges { target: addCity; visible: true }
        }
    ]

    tools: ToolbarActions {
        id: toolbarClock

        Action {
            id: addCity

            iconSource: Qt.resolvedUrl("../images/add_icon.png")
            text: i18n.tr("Add City")
            visible: true;

            onTriggered: clockPage.state = "ADDCITY";
        }

        back {
            visible: true;
            onTriggered: {
                clockPage.state = "NORMAL";
                worldClockList.clearUserSearch();
            }
        }
    }
}
