/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Alessandro Pozzi <signor.hyde@gmail.com>
 *              Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 *              Renato Araujo Oliveira Filho <renato.filho@canonical.com>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import "../common/Constants.js" as Constants
import "../common/ClockUtils.js" as Utils
import "StopwatchSupport.js" as SScript
import "../common"

Page {
    id: stopwatchPage

    property alias lapStart: laps.lapStart

    Component.onCompleted: {
        Utils.log("StopwatchPage loaded");
    }

    // Timer which runs every 100 microseconds to update the stopwatch label
    Timer {
        id: stopWatchLabelUpdate;

        interval: 100;
        repeat: true;
        onTriggered: {
            labelStopwatch.text = SScript.intervalToString(analogStopwatch.hours, analogStopwatch.minutes, analogStopwatch.seconds, analogStopwatch.milliseconds)
        }
    }

    anchors.margins: units.gu(2)

    ListModel {
        id: laps        

        property int lapStart: 0;

        function push() {
            insert(0, {"time" : (analogStopwatch.totalTime - lapStart),
                       "label": SScript.currentLapLabel(analogStopwatch.totalTime, lapStart)})
            lapStart = analogStopwatch.totalTime
        }
    }

    Label {
        id: labelStopwatch
        objectName: "labelStopwatch"

        anchors { left: parent.left; top: parent.top; topMargin: units.gu(5); right: parent.right }
        height: units.gu(6)
        horizontalAlignment: Text.AlignHCenter
        verticalAlignment: Text.AlignVCenter

        fontSize: "x-large"
        text: labelStopwatch.text = SScript.intervalToString(analogStopwatch.hours, analogStopwatch.minutes, analogStopwatch.seconds, analogStopwatch.milliseconds)
    }

    // Items within this container will move up/down depending on the state.
    AnimationContainer {
        id: stopwatchAnimationContainer

        // FIXME: The y is given a arbitary value which needs to be fixed by ensuring that the outerglow is not drawn outside the clock face area.
        initYPos: units.gu(5);
        finalYPos: -listLap.height;

        anchors { left: parent.left; top: labelStopwatch.bottom; right: parent.right; bottom: parent.bottom }

        // Component which draws the analog stopwatch face, controls the timer and calculates the time.
        AnalogStopwatch {
            id: analogStopwatch

            anchors { top: parent.top; horizontalCenter: parent.horizontalCenter }
        }

        // Bottom row to display Start, Reset buttons and lap information.
        Row {
            id: rowButtonStopwatch

            spacing: units.gu(2)
            anchors { top: analogStopwatch.bottom; topMargin: units.gu(8); horizontalCenter: parent.horizontalCenter }
            height: childrenRect.height

            Button {
                id: buttonStopwatch
                objectName: "buttonStopwatch"

                width: units.gu(12); height: units.gu(5)

                color: Constants.green
                text: i18n.tr("Start")

                onClicked: {
                    if(analogStopwatch.timerStatus == false) {
                        analogStopwatch.start()
                        stopWatchLabelUpdate.start()
                        if (analogStopwatch.flag) {
                            analogStopwatch.startTime = SScript.timeToMs(new Date());
                            analogStopwatch.flag = false;
                        }
                        else {
                            analogStopwatch.delay = Math.abs(SScript.timeToMs(new Date()) - analogStopwatch.stopTime);
                            console.log(analogStopwatch.startTime + "| " + analogStopwatch.delay);
                            analogStopwatch.startTime = analogStopwatch.startTime + analogStopwatch.delay;
                            console.log(analogStopwatch.startTime);
                        }
                        text = i18n.tr("Stop");
                        color = Constants.ubuntuOrange;
                        buttonReset.state = "LAP";
                    } else {
                        analogStopwatch.stop()
                        stopWatchLabelUpdate.stop()
                        analogStopwatch.stopTime = SScript.timeToMs(new Date());
                        console.log(analogStopwatch.stopTime)
                        text = i18n.tr("Start");
                        color = Constants.green;
                        buttonReset.state = "RESET";
                    }
                }
            }

            AnimatedLabel {
                id: lapCount
                objectName: "lapCount"

                anchors.verticalCenter: buttonStopwatch.verticalCenter
                color: Constants.brightWhite
                textColor: Constants.ubuntuOrange
                text: Utils.numberToLocaleString(laps.count)

                onClicked: stopwatchAnimationContainer.swipeView()
            }

            Button {
                id: buttonReset
                objectName: "lapButton"

                width: units.gu(12); height: units.gu(5)
                color: Constants.normalGrey
                text: i18n.tr("Reset")

                onClicked: {
                    if(buttonReset.state == "RESET") {
                        analogStopwatch.reset();
                        labelStopwatch.text = SScript.intervalToString(0, 0, 0, 0);
                        lapStart = 0;
                        laps.clear();
                    } else if (buttonReset.state == "LAP") {
                        laps.push();
                        analogStopwatch.blipLocation = analogStopwatch.seconds
                        analogStopwatch.blipVisible = true
                    }
                }

                states: [
                    State {
                        name: "RESET"
                        PropertyChanges { target: buttonReset; text: i18n.tr("Reset") }
                        PropertyChanges { target: buttonReset; color: Constants.normalGrey }
                    },

                    State {
                        name: "LAP"
                        PropertyChanges { target: buttonReset; text: i18n.tr("Lap") }
                        PropertyChanges { target: buttonReset; color: Constants.brightWhite }
                    }
                ]
            }
        }

        // ListView to display the current lap and recorded lap information.
        Column {
            id: listLap

            height: childrenRect.height
            anchors { left: parent.left; right: parent.right; top: rowButtonStopwatch.bottom; topMargin: units.gu(5) }

            ListItem.Divider { }

            ListItem.Header {
                id: lapHeader

                text: i18n.tr("Laps")
            }

            ListItem.SingleValue {
                id: currentLapItem

                text: i18n.tr("Current")
                value: SScript.currentLapLabel(analogStopwatch.totalTime, laps.lapStart)
            }

            ListItem.Header {
                id: currentLapHeader

                text: i18n.tr("Recorded laps")
            }

            ListView {
                id: listViewLap

                clip: true
                anchors { left: parent.left; right: parent.right; }
                height: units.gu(23)
                model: laps

                delegate: ListItem.SingleValue {
                    text: Utils.numberToLocaleString(laps.count - index)
                    value: model.label
                }
            }
        }
    }
}
